# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .services.org_policy import OrgPolicyClient
from .services.org_policy import OrgPolicyAsyncClient

from .types.constraint import Constraint
from .types.orgpolicy import AlternatePolicySpec
from .types.orgpolicy import CreatePolicyRequest
from .types.orgpolicy import DeletePolicyRequest
from .types.orgpolicy import GetEffectivePolicyRequest
from .types.orgpolicy import GetPolicyRequest
from .types.orgpolicy import ListConstraintsRequest
from .types.orgpolicy import ListConstraintsResponse
from .types.orgpolicy import ListPoliciesRequest
from .types.orgpolicy import ListPoliciesResponse
from .types.orgpolicy import Policy
from .types.orgpolicy import PolicySpec
from .types.orgpolicy import UpdatePolicyRequest

__all__ = (
    "OrgPolicyAsyncClient",
    "AlternatePolicySpec",
    "Constraint",
    "CreatePolicyRequest",
    "DeletePolicyRequest",
    "GetEffectivePolicyRequest",
    "GetPolicyRequest",
    "ListConstraintsRequest",
    "ListConstraintsResponse",
    "ListPoliciesRequest",
    "ListPoliciesResponse",
    "OrgPolicyClient",
    "Policy",
    "PolicySpec",
    "UpdatePolicyRequest",
)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Generated code. DO NOT EDIT!
#
# Snippet for CreatePolicy
# NOTE: This snippet has been automatically generated for illustrative purposes only.
# It may require modifications to work in your environment.

# To install the latest published package dependency, execute the following:
#   python3 -m pip install google-cloud-orgpolicy


# [START orgpolicy_v2_generated_OrgPolicy_CreatePolicy_sync]
from google.cloud import orgpolicy_v2


def sample_create_policy():
    # Create a client
    client = orgpolicy_v2.OrgPolicyClient()

    # Initialize request argument(s)
    request = orgpolicy_v2.CreatePolicyRequest(
        parent="parent_value",
    )

    # Make the request
    response = client.create_policy(request=request)

    # Handle the response
    print(response)

# [END orgpolicy_v2_generated_OrgPolicy_CreatePolicy_sync]



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.orgpolicy_v2.types import constraint
from google.protobuf import timestamp_pb2  # type: ignore
from google.type import expr_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.orgpolicy.v2",
    manifest={
        "Policy",
        "AlternatePolicySpec",
        "PolicySpec",
        "ListConstraintsRequest",
        "ListConstraintsResponse",
        "ListPoliciesRequest",
        "ListPoliciesResponse",
        "GetPolicyRequest",
        "GetEffectivePolicyRequest",
        "CreatePolicyRequest",
        "UpdatePolicyRequest",
        "DeletePolicyRequest",
    },
)


class Policy(proto.Message):
    r"""Defines a Cloud Organization ``Policy`` which is used to specify
    ``Constraints`` for configurations of Cloud Platform resources.

    Attributes:
        name (str):
            Immutable. The resource name of the Policy. Must be one of
            the following forms, where constraint_name is the name of
            the constraint which this Policy configures:

            -  ``projects/{project_number}/policies/{constraint_name}``
            -  ``folders/{folder_id}/policies/{constraint_name}``
            -  ``organizations/{organization_id}/policies/{constraint_name}``

            For example,
            "projects/123/policies/compute.disableSerialPortAccess".

            Note: ``projects/{project_id}/policies/{constraint_name}``
            is also an acceptable name for API requests, but responses
            will return the name using the equivalent project number.
        spec (google.cloud.orgpolicy_v2.types.PolicySpec):
            Basic information about the Organization
            Policy.
        alternate (google.cloud.orgpolicy_v2.types.AlternatePolicySpec):
            Deprecated.
    """

    name = proto.Field(
        proto.STRING,
        number=1,
    )
    spec = proto.Field(
        proto.MESSAGE,
        number=2,
        message="PolicySpec",
    )
    alternate = proto.Field(
        proto.MESSAGE,
        number=3,
        message="AlternatePolicySpec",
    )


class AlternatePolicySpec(proto.Message):
    r"""Similar to PolicySpec but with an extra 'launch' field for
    launch reference. The PolicySpec here is specific for
    dry-run/darklaunch.

    Attributes:
        launch (str):
            Reference to the launch that will be used
            while audit logging and to control the launch.
            Should be set only in the alternate policy.
        spec (google.cloud.orgpolicy_v2.types.PolicySpec):
            Specify ``Constraint`` for configurations of Cloud Platform
            resources.
    """

    launch = proto.Field(
        proto.STRING,
        number=1,
    )
    spec = proto.Field(
        proto.MESSAGE,
        number=2,
        message="PolicySpec",
    )


class PolicySpec(proto.Message):
    r"""Defines a Cloud Organization ``PolicySpec`` which is used to specify
    ``Constraints`` for configurations of Cloud Platform resources.

    Attributes:
        etag (str):
            An opaque tag indicating the current version of the
            ``Policy``, used for concurrency control.

            This field is ignored if used in a ``CreatePolicy`` request.

            When the ``Policy`` is returned from either a ``GetPolicy``
            or a ``ListPolicies`` request, this ``etag`` indicates the
            version of the current ``Policy`` to use when executing a
            read-modify-write loop.

            When the ``Policy`` is returned from a
            ``GetEffectivePolicy`` request, the ``etag`` will be unset.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time stamp this was previously updated.
            This represents the last time a call to ``CreatePolicy`` or
            ``UpdatePolicy`` was made for that ``Policy``.
        rules (Sequence[google.cloud.orgpolicy_v2.types.PolicySpec.PolicyRule]):
            Up to 10 PolicyRules are allowed.

            In Policies for boolean constraints, the following
            requirements apply:

            -  There must be one and only one PolicyRule where condition
               is unset.
            -  BooleanPolicyRules with conditions must set ``enforced``
               to the opposite of the PolicyRule without a condition.
            -  During policy evaluation, PolicyRules with conditions
               that are true for a target resource take precedence.
        inherit_from_parent (bool):
            Determines the inheritance behavior for this ``Policy``.

            If ``inherit_from_parent`` is true, PolicyRules set higher
            up in the hierarchy (up to the closest root) are inherited
            and present in the effective policy. If it is false, then no
            rules are inherited, and this Policy becomes the new root
            for evaluation. This field can be set only for Policies
            which configure list constraints.
        reset (bool):
            Ignores policies set above this resource and restores the
            ``constraint_default`` enforcement behavior of the specific
            ``Constraint`` at this resource. This field can be set in
            policies for either list or boolean constraints. If set,
            ``rules`` must be empty and ``inherit_from_parent`` must be
            set to false.
    """

    class PolicyRule(proto.Message):
        r"""A rule used to express this policy.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            values (google.cloud.orgpolicy_v2.types.PolicySpec.PolicyRule.StringValues):
                List of values to be used for this
                PolicyRule. This field can be set only in
                Policies for list constraints.

                This field is a member of `oneof`_ ``kind``.
            allow_all (bool):
                Setting this to true means that all values
                are allowed. This field can be set only in
                Policies for list constraints.

                This field is a member of `oneof`_ ``kind``.
            deny_all (bool):
                Setting this to true means that all values
                are denied. This field can be set only in
                Policies for list constraints.

                This field is a member of `oneof`_ ``kind``.
            enforce (bool):
                If ``true``, then the ``Policy`` is enforced. If ``false``,
                then any configuration is acceptable. This field can be set
                only in Policies for boolean constraints.

                This field is a member of `oneof`_ ``kind``.
            condition (google.type.expr_pb2.Expr):
                A condition which determines whether this rule is used in
                the evaluation of the policy. When set, the ``expression``
                field in the \`Expr' must include from 1 to 10
                subexpressions, joined by the "||" or "&&" operators. Each
                subexpression must be of the form
                "resource.matchTag('<ORG_ID>/tag_key_short_name,
                'tag_value_short_name')". or
                "resource.matchTagId('tagKeys/key_id',
                'tagValues/value_id')". where key_name and value_name are
                the resource names for Label Keys and Values. These names
                are available from the Tag Manager Service. An example
                expression is: "resource.matchTag('123456789/environment,
                'prod')". or "resource.matchTagId('tagKeys/123',
                'tagValues/456')".
        """

        class StringValues(proto.Message):
            r"""A message that holds specific allowed and denied values. This
            message can define specific values and subtrees of Cloud Resource
            Manager resource hierarchy (``Organizations``, ``Folders``,
            ``Projects``) that are allowed or denied. This is achieved by using
            the ``under:`` and optional ``is:`` prefixes. The ``under:`` prefix
            is used to denote resource subtree values. The ``is:`` prefix is
            used to denote specific values, and is required only if the value
            contains a ":". Values prefixed with "is:" are treated the same as
            values with no prefix. Ancestry subtrees must be in one of the
            following formats: - "projects/", e.g. "projects/tokyo-rain-123" -
            "folders/", e.g. "folders/1234" - "organizations/", e.g.
            "organizations/1234" The ``supports_under`` field of the associated
            ``Constraint`` defines whether ancestry prefixes can be used.

            Attributes:
                allowed_values (Sequence[str]):
                    List of values allowed at this resource.
                denied_values (Sequence[str]):
                    List of values denied at this resource.
            """

            allowed_values = proto.RepeatedField(
                proto.STRING,
                number=1,
            )
            denied_values = proto.RepeatedField(
                proto.STRING,
                number=2,
            )

        values = proto.Field(
            proto.MESSAGE,
            number=1,
            oneof="kind",
            message="PolicySpec.PolicyRule.StringValues",
        )
        allow_all = proto.Field(
            proto.BOOL,
            number=2,
            oneof="kind",
        )
        deny_all = proto.Field(
            proto.BOOL,
            number=3,
            oneof="kind",
        )
        enforce = proto.Field(
            proto.BOOL,
            number=4,
            oneof="kind",
        )
        condition = proto.Field(
            proto.MESSAGE,
            number=5,
            message=expr_pb2.Expr,
        )

    etag = proto.Field(
        proto.STRING,
        number=1,
    )
    update_time = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    rules = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=PolicyRule,
    )
    inherit_from_parent = proto.Field(
        proto.BOOL,
        number=4,
    )
    reset = proto.Field(
        proto.BOOL,
        number=5,
    )


class ListConstraintsRequest(proto.Message):
    r"""The request sent to the [ListConstraints]
    [google.cloud.orgpolicy.v2.OrgPolicy.ListConstraints] method.

    Attributes:
        parent (str):
            Required. The Cloud resource that parents the constraint.
            Must be in one of the following forms:

            -  ``projects/{project_number}``
            -  ``projects/{project_id}``
            -  ``folders/{folder_id}``
            -  ``organizations/{organization_id}``
        page_size (int):
            Size of the pages to be returned. This is
            currently unsupported and will be ignored. The
            server may at any point start using this field
            to limit page size.
        page_token (str):
            Page token used to retrieve the next page.
            This is currently unsupported and will be
            ignored. The server may at any point start using
            this field.
    """

    parent = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token = proto.Field(
        proto.STRING,
        number=3,
    )


class ListConstraintsResponse(proto.Message):
    r"""The response returned from the [ListConstraints]
    [google.cloud.orgpolicy.v2.OrgPolicy.ListConstraints] method.

    Attributes:
        constraints (Sequence[google.cloud.orgpolicy_v2.types.Constraint]):
            The collection of constraints that are
            available on the targeted resource.
        next_page_token (str):
            Page token used to retrieve the next page.
            This is currently not used.
    """

    @property
    def raw_page(self):
        return self

    constraints = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=constraint.Constraint,
    )
    next_page_token = proto.Field(
        proto.STRING,
        number=2,
    )


class ListPoliciesRequest(proto.Message):
    r"""The request sent to the [ListPolicies]
    [google.cloud.orgpolicy.v2.OrgPolicy.ListPolicies] method.

    Attributes:
        parent (str):
            Required. The target Cloud resource that parents the set of
            constraints and policies that will be returned from this
            call. Must be in one of the following forms:

            -  ``projects/{project_number}``
            -  ``projects/{project_id}``
            -  ``folders/{folder_id}``
            -  ``organizations/{organization_id}``
        page_size (int):
            Size of the pages to be returned. This is
            currently unsupported and will be ignored. The
            server may at any point start using this field
            to limit page size.
        page_token (str):
            Page token used to retrieve the next page.
            This is currently unsupported and will be
            ignored. The server may at any point start using
            this field.
    """

    parent = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token = proto.Field(
        proto.STRING,
        number=3,
    )


class ListPoliciesResponse(proto.Message):
    r"""The response returned from the [ListPolicies]
    [google.cloud.orgpolicy.v2.OrgPolicy.ListPolicies] method. It will
    be empty if no ``Policies`` are set on the resource.

    Attributes:
        policies (Sequence[google.cloud.orgpolicy_v2.types.Policy]):
            All ``Policies`` that exist on the resource. It will be
            empty if no ``Policies`` are set.
        next_page_token (str):
            Page token used to retrieve the next page.
            This is currently not used, but the server may
            at any point start supplying a valid token.
    """

    @property
    def raw_page(self):
        return self

    policies = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Policy",
    )
    next_page_token = proto.Field(
        proto.STRING,
        number=2,
    )


class GetPolicyRequest(proto.Message):
    r"""The request sent to the [GetPolicy]
    [google.cloud.orgpolicy.v2.OrgPolicy.GetPolicy] method.

    Attributes:
        name (str):
            Required. Resource name of the policy. See ``Policy`` for
            naming requirements.
    """

    name = proto.Field(
        proto.STRING,
        number=1,
    )


class GetEffectivePolicyRequest(proto.Message):
    r"""The request sent to the [GetEffectivePolicy]
    [google.cloud.orgpolicy.v2.OrgPolicy.GetEffectivePolicy] method.

    Attributes:
        name (str):
            Required. The effective policy to compute. See ``Policy``
            for naming rules.
    """

    name = proto.Field(
        proto.STRING,
        number=1,
    )


class CreatePolicyRequest(proto.Message):
    r"""The request sent to the [CreatePolicyRequest]
    [google.cloud.orgpolicy.v2.OrgPolicy.CreatePolicy] method.

    Attributes:
        parent (str):
            Required. The Cloud resource that will parent the new
            Policy. Must be in one of the following forms:

            -  ``projects/{project_number}``
            -  ``projects/{project_id}``
            -  ``folders/{folder_id}``
            -  ``organizations/{organization_id}``
        policy (google.cloud.orgpolicy_v2.types.Policy):
            Required. ``Policy`` to create.
    """

    parent = proto.Field(
        proto.STRING,
        number=1,
    )
    policy = proto.Field(
        proto.MESSAGE,
        number=3,
        message="Policy",
    )


class UpdatePolicyRequest(proto.Message):
    r"""The request sent to the [UpdatePolicyRequest]
    [google.cloud.orgpolicy.v2.OrgPolicy.UpdatePolicy] method.

    Attributes:
        policy (google.cloud.orgpolicy_v2.types.Policy):
            Required. ``Policy`` to update.
    """

    policy = proto.Field(
        proto.MESSAGE,
        number=1,
        message="Policy",
    )


class DeletePolicyRequest(proto.Message):
    r"""The request sent to the [DeletePolicy]
    [google.cloud.orgpolicy.v2.OrgPolicy.DeletePolicy] method.

    Attributes:
        name (str):
            Required. Name of the policy to delete. See ``Policy`` for
            naming rules.
    """

    name = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import warnings
from typing import Callable, Dict, Optional, Sequence, Tuple, Union

from google.api_core import grpc_helpers
from google.api_core import gapic_v1
import google.auth  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore

import grpc  # type: ignore

from google.cloud.orgpolicy_v2.types import orgpolicy
from google.protobuf import empty_pb2  # type: ignore
from .base import OrgPolicyTransport, DEFAULT_CLIENT_INFO


class OrgPolicyGrpcTransport(OrgPolicyTransport):
    """gRPC backend transport for OrgPolicy.

    An interface for managing organization policies.

    The Cloud Org Policy service provides a simple mechanism for
    organizations to restrict the allowed configurations across their
    entire Cloud Resource hierarchy.

    You can use a ``policy`` to configure restrictions in Cloud
    resources. For example, you can enforce a ``policy`` that restricts
    which Google Cloud Platform APIs can be activated in a certain part
    of your resource hierarchy, or prevents serial port access to VM
    instances in a particular folder.

    ``Policies`` are inherited down through the resource hierarchy. A
    ``policy`` applied to a parent resource automatically applies to all
    its child resources unless overridden with a ``policy`` lower in the
    hierarchy.

    A ``constraint`` defines an aspect of a resource's configuration
    that can be controlled by an organization's policy administrator.
    ``Policies`` are a collection of ``constraints`` that defines their
    allowable configuration on a particular resource and its child
    resources.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _stubs: Dict[str, Callable]

    def __init__(
        self,
        *,
        host: str = "orgpolicy.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        credentials_file: str = None,
        scopes: Sequence[str] = None,
        channel: grpc.Channel = None,
        api_mtls_endpoint: str = None,
        client_cert_source: Callable[[], Tuple[bytes, bytes]] = None,
        ssl_channel_credentials: grpc.ChannelCredentials = None,
        client_cert_source_for_mtls: Callable[[], Tuple[bytes, bytes]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if ``channel`` is provided.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            channel (Optional[grpc.Channel]): A ``Channel`` instance through
                which to make calls.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for the grpc channel. It is ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure a mutual TLS channel. It is
                ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.

        Raises:
          google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if channel:
            # Ignore credentials if a channel was passed.
            credentials = False
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None

        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )

        if not self._grpc_channel:
            self._grpc_channel = type(self).create_channel(
                self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                # Set ``credentials_file`` to ``None`` here as
                # the credentials that we saved earlier should be used.
                credentials_file=None,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        # Wrap messages. This must be done after self._grpc_channel exists
        self._prep_wrapped_messages(client_info)

    @classmethod
    def create_channel(
        cls,
        host: str = "orgpolicy.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        credentials_file: str = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> grpc.Channel:
        """Create and return a gRPC channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is mutually exclusive with credentials.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            grpc.Channel: A gRPC channel object.

        Raises:
            google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """

        return grpc_helpers.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            default_scopes=cls.AUTH_SCOPES,
            scopes=scopes,
            default_host=cls.DEFAULT_HOST,
            **kwargs,
        )

    @property
    def grpc_channel(self) -> grpc.Channel:
        """Return the channel designed to connect to this service."""
        return self._grpc_channel

    @property
    def list_constraints(
        self,
    ) -> Callable[
        [orgpolicy.ListConstraintsRequest], orgpolicy.ListConstraintsResponse
    ]:
        r"""Return a callable for the list constraints method over gRPC.

        Lists ``Constraints`` that could be applied on the specified
        resource.

        Returns:
            Callable[[~.ListConstraintsRequest],
                    ~.ListConstraintsResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_constraints" not in self._stubs:
            self._stubs["list_constraints"] = self.grpc_channel.unary_unary(
                "/google.cloud.orgpolicy.v2.OrgPolicy/ListConstraints",
                request_serializer=orgpolicy.ListConstraintsRequest.serialize,
                response_deserializer=orgpolicy.ListConstraintsResponse.deserialize,
            )
        return self._stubs["list_constraints"]

    @property
    def list_policies(
        self,
    ) -> Callable[[orgpolicy.ListPoliciesRequest], orgpolicy.ListPoliciesResponse]:
        r"""Return a callable for the list policies method over gRPC.

        Retrieves all of the ``Policies`` that exist on a particular
        resource.

        Returns:
            Callable[[~.ListPoliciesRequest],
                    ~.ListPoliciesResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_policies" not in self._stubs:
            self._stubs["list_policies"] = self.grpc_channel.unary_unary(
                "/google.cloud.orgpolicy.v2.OrgPolicy/ListPolicies",
                request_serializer=orgpolicy.ListPoliciesRequest.serialize,
                response_deserializer=orgpolicy.ListPoliciesResponse.deserialize,
            )
        return self._stubs["list_policies"]

    @property
    def get_policy(self) -> Callable[[orgpolicy.GetPolicyRequest], orgpolicy.Policy]:
        r"""Return a callable for the get policy method over gRPC.

        Gets a ``Policy`` on a resource.

        If no ``Policy`` is set on the resource, NOT_FOUND is returned.
        The ``etag`` value can be used with ``UpdatePolicy()`` to update
        a ``Policy`` during read-modify-write.

        Returns:
            Callable[[~.GetPolicyRequest],
                    ~.Policy]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_policy" not in self._stubs:
            self._stubs["get_policy"] = self.grpc_channel.unary_unary(
                "/google.cloud.orgpolicy.v2.OrgPolicy/GetPolicy",
                request_serializer=orgpolicy.GetPolicyRequest.serialize,
                response_deserializer=orgpolicy.Policy.deserialize,
            )
        return self._stubs["get_policy"]

    @property
    def get_effective_policy(
        self,
    ) -> Callable[[orgpolicy.GetEffectivePolicyRequest], orgpolicy.Policy]:
        r"""Return a callable for the get effective policy method over gRPC.

        Gets the effective ``Policy`` on a resource. This is the result
        of merging ``Policies`` in the resource hierarchy and evaluating
        conditions. The returned ``Policy`` will not have an ``etag`` or
        ``condition`` set because it is a computed ``Policy`` across
        multiple resources. Subtrees of Resource Manager resource
        hierarchy with 'under:' prefix will not be expanded.

        Returns:
            Callable[[~.GetEffectivePolicyRequest],
                    ~.Policy]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_effective_policy" not in self._stubs:
            self._stubs["get_effective_policy"] = self.grpc_channel.unary_unary(
                "/google.cloud.orgpolicy.v2.OrgPolicy/GetEffectivePolicy",
                request_serializer=orgpolicy.GetEffectivePolicyRequest.serialize,
                response_deserializer=orgpolicy.Policy.deserialize,
            )
        return self._stubs["get_effective_policy"]

    @property
    def create_policy(
        self,
    ) -> Callable[[orgpolicy.CreatePolicyRequest], orgpolicy.Policy]:
        r"""Return a callable for the create policy method over gRPC.

        Creates a Policy.

        Returns a ``google.rpc.Status`` with
        ``google.rpc.Code.NOT_FOUND`` if the constraint does not exist.
        Returns a ``google.rpc.Status`` with
        ``google.rpc.Code.ALREADY_EXISTS`` if the policy already exists
        on the given Cloud resource.

        Returns:
            Callable[[~.CreatePolicyRequest],
                    ~.Policy]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_policy" not in self._stubs:
            self._stubs["create_policy"] = self.grpc_channel.unary_unary(
                "/google.cloud.orgpolicy.v2.OrgPolicy/CreatePolicy",
                request_serializer=orgpolicy.CreatePolicyRequest.serialize,
                response_deserializer=orgpolicy.Policy.deserialize,
            )
        return self._stubs["create_policy"]

    @property
    def update_policy(
        self,
    ) -> Callable[[orgpolicy.UpdatePolicyRequest], orgpolicy.Policy]:
        r"""Return a callable for the update policy method over gRPC.

        Updates a Policy.

        Returns a ``google.rpc.Status`` with
        ``google.rpc.Code.NOT_FOUND`` if the constraint or the policy do
        not exist. Returns a ``google.rpc.Status`` with
        ``google.rpc.Code.ABORTED`` if the etag supplied in the request
        does not match the persisted etag of the policy

        Note: the supplied policy will perform a full overwrite of all
        fields.

        Returns:
            Callable[[~.UpdatePolicyRequest],
                    ~.Policy]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_policy" not in self._stubs:
            self._stubs["update_policy"] = self.grpc_channel.unary_unary(
                "/google.cloud.orgpolicy.v2.OrgPolicy/UpdatePolicy",
                request_serializer=orgpolicy.UpdatePolicyRequest.serialize,
                response_deserializer=orgpolicy.Policy.deserialize,
            )
        return self._stubs["update_policy"]

    @property
    def delete_policy(
        self,
    ) -> Callable[[orgpolicy.DeletePolicyRequest], empty_pb2.Empty]:
        r"""Return a callable for the delete policy method over gRPC.

        Deletes a Policy.

        Returns a ``google.rpc.Status`` with
        ``google.rpc.Code.NOT_FOUND`` if the constraint or Org Policy
        does not exist.

        Returns:
            Callable[[~.DeletePolicyRequest],
                    ~.Empty]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_policy" not in self._stubs:
            self._stubs["delete_policy"] = self.grpc_channel.unary_unary(
                "/google.cloud.orgpolicy.v2.OrgPolicy/DeletePolicy",
                request_serializer=orgpolicy.DeletePolicyRequest.serialize,
                response_deserializer=empty_pb2.Empty.FromString,
            )
        return self._stubs["delete_policy"]

    def close(self):
        self.grpc_channel.close()

    @property
    def kind(self) -> str:
        return "grpc"


__all__ = ("OrgPolicyGrpcTransport",)



# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/cloud/orgpolicy/v1/orgpolicy.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n)google/cloud/orgpolicy/v1/orgpolicy.proto\x12\x19google.cloud.orgpolicy.v1\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/api/annotations.proto"\x8b\x05\n\x06Policy\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x12\n\nconstraint\x18\x02 \x01(\t\x12\x0c\n\x04\x65tag\x18\x03 \x01(\x0c\x12/\n\x0bupdate_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x43\n\x0blist_policy\x18\x05 \x01(\x0b\x32,.google.cloud.orgpolicy.v1.Policy.ListPolicyH\x00\x12I\n\x0e\x62oolean_policy\x18\x06 \x01(\x0b\x32/.google.cloud.orgpolicy.v1.Policy.BooleanPolicyH\x00\x12K\n\x0frestore_default\x18\x07 \x01(\x0b\x32\x30.google.cloud.orgpolicy.v1.Policy.RestoreDefaultH\x00\x1a\xfb\x01\n\nListPolicy\x12\x16\n\x0e\x61llowed_values\x18\x01 \x03(\t\x12\x15\n\rdenied_values\x18\x02 \x03(\t\x12J\n\nall_values\x18\x03 \x01(\x0e\x32\x36.google.cloud.orgpolicy.v1.Policy.ListPolicy.AllValues\x12\x17\n\x0fsuggested_value\x18\x04 \x01(\t\x12\x1b\n\x13inherit_from_parent\x18\x05 \x01(\x08"<\n\tAllValues\x12\x1a\n\x16\x41LL_VALUES_UNSPECIFIED\x10\x00\x12\t\n\x05\x41LLOW\x10\x01\x12\x08\n\x04\x44\x45NY\x10\x02\x1a!\n\rBooleanPolicy\x12\x10\n\x08\x65nforced\x18\x01 \x01(\x08\x1a\x10\n\x0eRestoreDefaultB\r\n\x0bpolicy_typeB\xcc\x01\n\x1d\x63om.google.cloud.orgpolicy.v1B\x0eOrgPolicyProtoP\x01ZBgoogle.golang.org/genproto/googleapis/cloud/orgpolicy/v1;orgpolicy\xaa\x02\x19Google.Cloud.OrgPolicy.V1\xca\x02\x19Google\\Cloud\\OrgPolicy\\V1\xea\x02\x1cGoogle::Cloud::OrgPolicy::V1b\x06proto3'
)


_POLICY = DESCRIPTOR.message_types_by_name["Policy"]
_POLICY_LISTPOLICY = _POLICY.nested_types_by_name["ListPolicy"]
_POLICY_BOOLEANPOLICY = _POLICY.nested_types_by_name["BooleanPolicy"]
_POLICY_RESTOREDEFAULT = _POLICY.nested_types_by_name["RestoreDefault"]
_POLICY_LISTPOLICY_ALLVALUES = _POLICY_LISTPOLICY.enum_types_by_name["AllValues"]
Policy = _reflection.GeneratedProtocolMessageType(
    "Policy",
    (_message.Message,),
    {
        "ListPolicy": _reflection.GeneratedProtocolMessageType(
            "ListPolicy",
            (_message.Message,),
            {
                "DESCRIPTOR": _POLICY_LISTPOLICY,
                "__module__": "google.cloud.orgpolicy.v1.orgpolicy_pb2"
                # @@protoc_insertion_point(class_scope:google.cloud.orgpolicy.v1.Policy.ListPolicy)
            },
        ),
        "BooleanPolicy": _reflection.GeneratedProtocolMessageType(
            "BooleanPolicy",
            (_message.Message,),
            {
                "DESCRIPTOR": _POLICY_BOOLEANPOLICY,
                "__module__": "google.cloud.orgpolicy.v1.orgpolicy_pb2"
                # @@protoc_insertion_point(class_scope:google.cloud.orgpolicy.v1.Policy.BooleanPolicy)
            },
        ),
        "RestoreDefault": _reflection.GeneratedProtocolMessageType(
            "RestoreDefault",
            (_message.Message,),
            {
                "DESCRIPTOR": _POLICY_RESTOREDEFAULT,
                "__module__": "google.cloud.orgpolicy.v1.orgpolicy_pb2"
                # @@protoc_insertion_point(class_scope:google.cloud.orgpolicy.v1.Policy.RestoreDefault)
            },
        ),
        "DESCRIPTOR": _POLICY,
        "__module__": "google.cloud.orgpolicy.v1.orgpolicy_pb2"
        # @@protoc_insertion_point(class_scope:google.cloud.orgpolicy.v1.Policy)
    },
)
_sym_db.RegisterMessage(Policy)
_sym_db.RegisterMessage(Policy.ListPolicy)
_sym_db.RegisterMessage(Policy.BooleanPolicy)
_sym_db.RegisterMessage(Policy.RestoreDefault)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"\n\035com.google.cloud.orgpolicy.v1B\016OrgPolicyProtoP\001ZBgoogle.golang.org/genproto/googleapis/cloud/orgpolicy/v1;orgpolicy\252\002\031Google.Cloud.OrgPolicy.V1\312\002\031Google\\Cloud\\OrgPolicy\\V1\352\002\034Google::Cloud::OrgPolicy::V1"
    _POLICY._serialized_start = 165
    _POLICY._serialized_end = 816
    _POLICY_LISTPOLICY._serialized_start = 497
    _POLICY_LISTPOLICY._serialized_end = 748
    _POLICY_LISTPOLICY_ALLVALUES._serialized_start = 688
    _POLICY_LISTPOLICY_ALLVALUES._serialized_end = 748
    _POLICY_BOOLEANPOLICY._serialized_start = 750
    _POLICY_BOOLEANPOLICY._serialized_end = 783
    _POLICY_RESTOREDEFAULT._serialized_start = 785
    _POLICY_RESTOREDEFAULT._serialized_end = 801
# @@protoc_insertion_point(module_scope)
